Package: BoomSpikeSlab
Version: 1.2.7
Date: 2025-09-03
Title: MCMC for Spike and Slab Regression
Authors@R: c(person(
               given=c("Steven", "L."),
               family = "Scott",
               role = c("aut", "cre"),
               email = "steve.the.bayesian@gmail.com"))
Description: Spike and slab regression with a variety of residual error
  distributions corresponding to Gaussian, Student T, probit, logit, SVM, and a
  few others.  Spike and slab regression is Bayesian regression with prior
  distributions containing a point mass at zero.  The posterior updates the
  amount of mass on this point, leading to a posterior distribution that is
  actually sparse, in the sense that if you sample from it many coefficients are
  actually zeros.  Sampling from this posterior distribution is an elegant way
  to handle Bayesian variable selection and model averaging.  See
  <DOI:10.1504/IJMMNO.2014.059942> for an explanation of the Gaussian case.
License: LGPL-2.1 | file LICENSE
Depends: Boom (>= 0.9.16) , R (>= 4.5.0)
LinkingTo: Boom(>= 0.9.16)
Suggests: MASS, testthat, mlbench, igraph
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2025-09-03 23:54:43 UTC; steve
Author: Steven L. Scott [aut, cre]
Maintainer: Steven L. Scott <steve.the.bayesian@gmail.com>
Repository: CRAN
Date/Publication: 2025-09-04 04:50:09 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-27 07:40:18 UTC; unix
Archs: BoomSpikeSlab.so.dSYM
